@@ -78,8 +78,6 @@ CREATE VIEW repmgr.show_nodes AS
 LEFT JOIN repmgr.nodes un
        ON un.node_id = n.upstream_node_id;
 
-
-/* XXX update upgrade scripts! */
 CREATE TABLE repmgr.voting_term (
   term INT NOT NULL
 );
@@ -118,6 +116,26 @@ CREATE FUNCTION standby_get_last_updated()
   AS 'MODULE_PATHNAME', 'standby_get_last_updated'
   LANGUAGE C STRICT;
 
+CREATE FUNCTION set_upstream_last_seen(INT)
+  RETURNS VOID
+  AS 'MODULE_PATHNAME', 'set_upstream_last_seen'
+  LANGUAGE C STRICT;
+
+CREATE FUNCTION get_upstream_last_seen()
+  RETURNS INT
+  AS 'MODULE_PATHNAME', 'get_upstream_last_seen'
+  LANGUAGE C STRICT;
+
+CREATE FUNCTION get_upstream_node_id()
+  RETURNS INT
+  AS 'MODULE_PATHNAME', 'get_upstream_node_id'
+  LANGUAGE C STRICT;
+
+CREATE FUNCTION set_upstream_node_id(INT)
+  RETURNS VOID
+  AS 'MODULE_PATHNAME', 'set_upstream_node_id'
+  LANGUAGE C STRICT;
+
 /* failover functions */
 
 CREATE FUNCTION notify_follow_primary(INT)
@@ -158,7 +176,7 @@ CREATE FUNCTION get_repmgrd_pidfile()
 CREATE FUNCTION set_repmgrd_pid(INT, TEXT)
   RETURNS VOID
   AS 'MODULE_PATHNAME', 'set_repmgrd_pid'
-  LANGUAGE C STRICT;
+  LANGUAGE C CALLED ON NULL INPUT;
 
 CREATE FUNCTION repmgrd_is_running()
   RETURNS BOOL
@@ -175,6 +193,15 @@ CREATE FUNCTION repmgrd_is_paused()
   AS 'MODULE_PATHNAME', 'repmgrd_is_paused'
   LANGUAGE C STRICT;
 
+CREATE FUNCTION get_wal_receiver_pid()
+  RETURNS INT
+  AS 'MODULE_PATHNAME', 'get_wal_receiver_pid'
+  LANGUAGE C STRICT;
+
+
+
+
+/* views */
 
 CREATE VIEW repmgr.replication_status AS
   SELECT m.primary_node_id, m.standby_node_id, n.node_name AS standby_name,
