@@ -19,10 +19,6 @@
 #ifndef _REPMGR_ACTION_DAEMON_H_
 #define _REPMGR_ACTION_DAEMON_H_
 
-
-extern void do_daemon_status(void);
-extern void do_daemon_pause(void);
-extern void do_daemon_unpause(void);
 extern void do_daemon_start(void);
 extern void do_daemon_stop(void);
 
