@@ -3,7 +3,7 @@
  *
  * Implements primary actions for the repmgr command line utility
  *
- * Copyright (c) 2ndQuadrant, 2010-2018
+ * Copyright (c) 2ndQuadrant, 2010-2019
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -96,28 +96,6 @@ do_primary_register(void)
 
 	initialize_voting_term(conn);
 
-	/* Ensure there isn't another registered node which is primary */
-	primary_conn = get_primary_connection(conn, &current_primary_id, NULL);
-
-	if (primary_conn != NULL)
-	{
-		if (current_primary_id != config_file_options.node_id)
-		{
-			/*
-			 * it's impossible to add a second primary to a streaming
-			 * replication cluster
-			 */
-			log_error(_("there is already an active registered primary (node ID: %i) in this cluster"), current_primary_id);
-			PQfinish(primary_conn);
-			PQfinish(conn);
-			exit(ERR_BAD_CONFIG);
-		}
-
-		/* we've probably connected to ourselves */
-		PQfinish(primary_conn);
-	}
-
-
 	begin_transaction(conn);
 
 	/*
@@ -128,12 +106,32 @@ do_primary_register(void)
 	current_primary_id = get_primary_node_id(conn);
 	if (current_primary_id != NODE_NOT_FOUND && current_primary_id != config_file_options.node_id)
 	{
-		log_error(_("another node with id %i is already registered as primary"), current_primary_id);
-		log_detail(_("a streaming replication cluster can have only one primary node"));
+		log_debug("XXX %i", current_primary_id);
+		primary_conn = establish_primary_db_connection(conn, false);
 
-		rollback_transaction(conn);
-		PQfinish(conn);
-		exit(ERR_BAD_CONFIG);
+		if (PQstatus(primary_conn) == CONNECTION_OK)
+		{
+			if (get_recovery_type(primary_conn) == RECTYPE_PRIMARY)
+			{
+				log_error(_("there is already an active registered primary (ID: %i) in this cluster"),
+						  current_primary_id);
+				log_detail(_("a streaming replication cluster can have only one primary node"));
+
+				log_hint(_("ensure this node is shut down before registering a new primary"));
+				PQfinish(primary_conn);
+				rollback_transaction(conn);
+				PQfinish(conn);
+				exit(ERR_BAD_CONFIG);
+			}
+
+			log_warning(_("node %is is registered as primary but running as a standby"),
+						  current_primary_id);
+			PQfinish(primary_conn);
+		}
+
+		log_notice(_("setting node %i's node record to inactive"),
+						  current_primary_id);
+		update_node_record_set_active(conn, current_primary_id, false);
 	}
 
 	/*
@@ -225,12 +223,12 @@ do_primary_register(void)
 
 	if (record_status == RECORD_FOUND)
 	{
-		log_notice(_("primary node record (id: %i) updated"),
+		log_notice(_("primary node record (ID: %i) updated"),
 				   config_file_options.node_id);
 	}
 	else
 	{
-		log_notice(_("primary node record (id: %i) registered"),
+		log_notice(_("primary node record (ID: %i) registered"),
 				   config_file_options.node_id);
 	}
 
@@ -251,6 +249,7 @@ do_primary_unregister(void)
 	PGconn	   *primary_conn = NULL;
 	PGconn	   *local_conn = NULL;
 	t_node_info local_node_info = T_NODE_INFO_INITIALIZER;
+	t_node_info primary_node_info = T_NODE_INFO_INITIALIZER;
 
 	t_node_info *target_node_info_ptr = NULL;
 	PGconn	   *target_node_conn = NULL;
@@ -271,13 +270,11 @@ do_primary_unregister(void)
 
 	if (PQstatus(primary_conn) != CONNECTION_OK)
 	{
-		t_node_info primary_node_info = T_NODE_INFO_INITIALIZER;
-
 		log_error(_("unable to connect to primary server"));
 
 		if (get_primary_node_record(local_conn, &primary_node_info) == true)
 		{
-			log_detail(_("current primary registered as node %s (id: %i, conninfo: \"%s\")"),
+			log_detail(_("current primary registered as node \"%s\" (ID: %i, conninfo: \"%s\")"),
 					   primary_node_info.node_name,
 					   primary_node_info.node_id,
 					   primary_node_info.conninfo);
@@ -291,10 +288,19 @@ do_primary_unregister(void)
 	/* Local connection no longer required */
 	PQfinish(local_conn);
 
+	if (get_primary_node_record(primary_conn, &primary_node_info) == false)
+	{
+		log_error(_("unable to retrieve record for primary node"));
+		PQfinish(primary_conn);
+		exit(ERR_BAD_CONFIG);
+	}
 
 	/* Target node is local node? */
-	if (target_node_info.node_id == UNKNOWN_NODE_ID
-		|| target_node_info.node_id == config_file_options.node_id)
+	if (target_node_info.node_id == UNKNOWN_NODE_ID)
+	{
+		target_node_info_ptr = &primary_node_info;
+	}
+	else if (target_node_info.node_id == config_file_options.node_id)
 	{
 		target_node_info_ptr = &local_node_info;
 	}
@@ -304,6 +310,24 @@ do_primary_unregister(void)
 		target_node_info_ptr = &target_node_info;
 	}
 
+	/*
+	 * Sanity-check the target node is not a witness
+	 */
+
+	if (target_node_info_ptr->type == WITNESS)
+	{
+		log_error(_("node \"%s\" (ID: %i) is a witness server, unable to unregister"),
+					  target_node_info_ptr->node_name,
+					  target_node_info_ptr->node_id);
+		if (target_node_info_ptr->type == STANDBY)
+		{
+			log_hint(_("the node can be unregistered with \"repmgr witness unregister\""));
+		}
+
+		PQfinish(primary_conn);
+		exit(ERR_BAD_CONFIG);
+	}
+
 	/*
 	 * Check for downstream nodes - if any still defined, we won't be able to
 	 * delete the node record due to foreign key constraints.
@@ -333,7 +357,7 @@ do_primary_unregister(void)
 		for (cell = downstream_nodes.head; cell; cell = cell->next)
 		{
 			appendPQExpBuffer(&detail,
-							  "  %s (id: %i)\n",
+							  "  %s (ID: %i)\n",
 							  cell->node_info->node_name,
 							  cell->node_info->node_id);
 		}
@@ -353,7 +377,7 @@ do_primary_unregister(void)
 	{
 		if (target_node_info_ptr->type != PRIMARY)
 		{
-			log_error(_("node %s (id: %i) is not a primary, unable to unregister"),
+			log_error(_("node \"%s\" (ID: %i) is not a primary, unable to unregister"),
 					  target_node_info_ptr->node_name,
 					  target_node_info_ptr->node_id);
 			if (target_node_info_ptr->type == STANDBY)
@@ -380,7 +404,7 @@ do_primary_unregister(void)
 			 */
 			if (target_node_info_ptr->type != PRIMARY)
 			{
-				log_error(_("node %s (ID: %i) is a %s, unable to unregister"),
+				log_error(_("node \"%s\" (ID: %i) is a %s, unable to unregister"),
 						  target_node_info_ptr->node_name,
 						  target_node_info_ptr->node_id,
 						  get_node_type_string(target_node_info_ptr->type));
@@ -394,7 +418,7 @@ do_primary_unregister(void)
 			 */
 			else if (!runtime_options.force)
 			{
-				log_error(_("node %s (ID: %i) is running as a standby, unable to unregister"),
+				log_error(_("node \"%s\" (ID: %i) is running as a standby, unable to unregister"),
 						  target_node_info_ptr->node_name,
 						  target_node_info_ptr->node_id);
 				log_hint(_("the node can be registered as a standby with \"repmgr standby register --force\""));
@@ -419,7 +443,7 @@ do_primary_unregister(void)
 
 			if (primary_record_found == false)
 			{
-				log_error(_("node %s (ID: %i) is a primary node, but no primary node record found"),
+				log_error(_("node \"%s\" (ID: %i) is a primary node, but no primary node record found"),
 						  target_node_info_ptr->node_name,
 						  target_node_info_ptr->node_id);
 				log_hint(_("register this node as primary with \"repmgr primary register --force\""));
@@ -434,7 +458,7 @@ do_primary_unregister(void)
 			 */
 			if (primary_node_info.node_id == target_node_info_ptr->node_id)
 			{
-				log_error(_("node %s (ID: %i) is the current primary node, unable to unregister"),
+				log_error(_("node \"%s\" (ID: %i) is the current primary node, unable to unregister"),
 						  target_node_info_ptr->node_name,
 						  target_node_info_ptr->node_id);
 
@@ -456,7 +480,7 @@ do_primary_unregister(void)
 	{
 		if (!runtime_options.force)
 		{
-			log_error(_("node %s (ID: %i) is marked as active, unable to unregister"),
+			log_error(_("node \"%s\" (ID: %i) is marked as active, unable to unregister"),
 					  target_node_info_ptr->node_name,
 					  target_node_info_ptr->node_id);
 			log_hint(_("run \"repmgr primary unregister --force\" to unregister this node"));
@@ -467,7 +491,7 @@ do_primary_unregister(void)
 
 	if (runtime_options.dry_run == true)
 	{
-		log_notice(_("node %s (ID: %i) would now be unregistered"),
+		log_notice(_("node \"%s\" (ID: %i) would now be unregistered"),
 				   target_node_info_ptr->node_name,
 				   target_node_info_ptr->node_id);
 		log_hint(_("run the same command without the --dry-run option to unregister this node"));
@@ -480,7 +504,7 @@ do_primary_unregister(void)
 
 		if (delete_success == false)
 		{
-			log_error(_("unable to unregister node %s (ID: %i)"),
+			log_error(_("unable to unregister node \"%s\" (ID: %i)"),
 					  target_node_info_ptr->node_name,
 					  target_node_info_ptr->node_id);
 			PQfinish(primary_conn);
@@ -489,14 +513,14 @@ do_primary_unregister(void)
 
 		initPQExpBuffer(&event_details);
 		appendPQExpBuffer(&event_details,
-						  _("node %s (ID: %i) unregistered"),
+						  _("node \"%s\" (ID: %i) unregistered"),
 						  target_node_info_ptr->node_name,
 						  target_node_info_ptr->node_id);
 
 		if (target_node_info_ptr->node_id != config_file_options.node_id)
 		{
 			appendPQExpBuffer(&event_details,
-							  _(" from node %s (ID: %i)"),
+							  _(" from node \"%s\" (ID: %i)"),
 							  config_file_options.node_name,
 							  config_file_options.node_id);
 		}
@@ -509,7 +533,7 @@ do_primary_unregister(void)
 								  event_details.data);
 		termPQExpBuffer(&event_details);
 
-		log_info(_("node %s (ID: %i) was successfully unregistered"),
+		log_info(_("node \"%s\" (ID: %i) was successfully unregistered"),
 				 target_node_info_ptr->node_name,
 				 target_node_info_ptr->node_id);
 	}
