@@ -21,6 +21,37 @@
 #include <config.h>
 #endif
 
+#ifdef vsnprintf
+#undef vsnprintf
+#endif
+#ifdef snprintf
+#undef snprintf
+#endif
+#ifdef vsprintf
+#undef vsprintf
+#endif
+#ifdef sprintf
+#undef sprintf
+#endif
+#ifdef vfprintf
+#undef vfprintf
+#endif
+#ifdef fprintf
+#undef fprintf
+#endif
+#ifdef vprintf
+#undef vprintf
+#endif
+#ifdef printf
+#undef printf
+#endif
+#ifdef strerror
+#undef strerror
+#endif
+#ifdef strerror_r
+#undef strerror_r
+#endif
+
 #ifndef _REPMGR_H_
 #define _REPMGR_H_
 
@@ -51,6 +82,7 @@
 
 #define UNKNOWN_SERVER_VERSION_NUM -1
 #define UNKNOWN_BDR_VERSION_NUM -1
+#define UNKNOWN_REPMGR_VERSION_NUM -1
 
 #define UNKNOWN_TIMELINE_ID -1
 #define UNKNOWN_SYSTEM_IDENTIFIER 0
@@ -108,6 +140,11 @@
 #define RECOVERY_COMMAND_FILE "recovery.conf"
 #endif
 
+#ifndef STANDBY_SIGNAL_FILE
+#define STANDBY_SIGNAL_FILE "standby.signal"
+#define RECOVERY_SIGNAL_FILE "recovery.signal"
+#endif
+
 #ifndef TABLESPACE_MAP
 #define TABLESPACE_MAP "tablespace_map"
 #endif
